# ABSTRACT: Dancer2 cli application
package Dancer2::CLI;
$Dancer2::CLI::VERSION = '0.149000_01';
use App::Cmd::Setup -app;

1;

__END__

=pod

=head1 NAME

Dancer2::CLI - Dancer2 cli application

=head1 VERSION

version 0.149000_01

=head1 AUTHOR

Dancer Core Developers

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Alexis Sukrieh.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
